# This is a namespace package so that `backports.datetime_fromisoformat` can integrate 
# nicely with the other `backports` modules.
# See https://packaging.python.org/guides/packaging-namespace-packages/#pkgutil-style-namespace-packages

__path__ = __import__('pkgutil').extend_path(__path__, __name__)


import os

from setuptools import setup, Extension
# workaround for open() with encoding='' python2/3 compability
from io import open

with open('README.rst', encoding='utf-8') as file:
    long_description = file.read()

VERSION = "1.0.0"

setup(
    name="backports-datetime-fromisoformat",
    version=VERSION,
    description="Backport of Python 3.7's datetime.fromisoformat",
    long_description=long_description,
    license="MIT",
    author="Michael Overmeyer",
    author_email="backports@movermeyer.com",
    url="https://github.com/movermeyer/backports.datetime_fromisoformat",

    packages=["backports", "backports.datetime_fromisoformat"],

    ext_modules=[Extension("backports._datetime_fromisoformat", [
        os.path.join("backports", "datetime_fromisoformat", "module.c"),
        os.path.join("backports", "datetime_fromisoformat", "_datetimemodule.c"),
        os.path.join("backports", "datetime_fromisoformat", "timezone.c")
    ])],

    test_suite='tests',
    tests_require=[
        'pytz'
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



import os
import sys

from backports._datetime_fromisoformat import date_fromisoformat, datetime_fromisoformat, time_fromisoformat, FixedOffset


class MonkeyPatch(object):
    @staticmethod
    def patch_fromisoformat():
        import ctypes as c

        def flush_mro_cache():
            # From https://stackoverflow.com/questions/24497316/set-a-read-only-attribute-in-python/24498525#24498525

            if os.name == "nt":
                pythonapi = c.PyDLL("python dll", None, sys.dllhandle)
            elif sys.platform == "cygwin":
                pythonapi = c.PyDLL("libpython%d.%d.dll" % sys.version_info[:2])
            else:
                pythonapi = c.PyDLL(None)

            pythonapi.PyType_Modified(c.py_object(object))

        _get_dict = c.pythonapi._PyObject_GetDictPtr
        _get_dict.restype = c.POINTER(c.py_object)
        _get_dict.argtypes = [c.py_object]

        from datetime import date, datetime, time

        try:
            _ = datetime.fromisoformat
        except AttributeError:
            d = _get_dict(datetime)[0]
            d['fromisoformat'] = datetime_fromisoformat

        try:
            _ = date.fromisoformat
        except AttributeError:
            d = _get_dict(date)[0]
            d['fromisoformat'] = date_fromisoformat

        try:
            _ = time.fromisoformat
        except AttributeError:
            d = _get_dict(time)[0]
            d['fromisoformat'] = time_fromisoformat

        flush_mro_cache()
